local fileSet_module = {}

local u_loadModuleFiles = require("modules/utils/loadModuleFiles")

-- order: #type(eg. bullet, item, melee)/name/data,more_data,moore

local TEMPLATES_FOLDER = "modules/file/templates"

local sets = {}
local templates = {}
local entryLookUp = {}

function fileSet_module.loadTemplates()
	templates = u_loadModuleFiles(TEMPLATES_FOLDER, "lua", "template")
end

function fileSet_module.loadSets(filepath)
	local lineCount = 1
	for line in love.filesystem.lines(filepath) do
		local entryExist, _, entryType, entryName, rawData = string.find(line, "^#(%w+)/(%w+)/(.*)")
		if entryExist then
			-- process arguments
			local finalData = {}
			local origin = 1
			while #rawData >= origin do
				local isTemplate, lastTemplate, templateVar = string.find(rawData, "^(%w+%b()),?", origin)
				if not isTemplate then -- single
					local _, lastSingle, singleVar = string.find(rawData, "^(%w+),?", origin)
					finalData[#finalData + 1] = singleVar
					origin = lastSingle + 1
				else -- template
					local _, _, templateName, templateArgs = string.find(templateVar, "(%w+)%((.-)%)")
					-- process template arguments
					local finalArgs = {}
					local argSlice = {}
					string.gsub(templateArgs, "(%w+)([,:]?)", function(arg, split)
						argSlice[#argSlice + 1] = arg
						if split ~= "," then
							finalArgs[#finalArgs + 1] = argSlice
							argSlice = {}
						end
					end)
					-- template code
					if templates[templateName] then
						finalData[#finalData + 1] = templates[templateName](finalArgs, entryType, entryName)
					else
						print("WARNING!\nTemplate type "..templateName.." is not defined\nLine: "..lineCount)
						finalData[#finalData + 1] = finalArgs
					end
					origin = lastTemplate + 1
				end
			end
			-- insert finalized data
			if not sets[entryType] then
				sets[entryType] = {}
			end
			sets[entryType][entryName] = finalData
			-- look up table
			if not entryLookUp[entryType] then
				entryLookUp[entryType] = {}
			end
			entryLookUp[entryType][#entryLookUp[entryType] + 1] = entryName
		end
		lineCount = lineCount + 1
	end
end

function fileSet_module.get(setName, entryName)
	return sets[setName][entryName]
end

function fileSet_module.getWhole(setName)
	return sets[setName]
end

function fileSet_module.getLen(setName)
	-- local l = 0
	-- for i, v in pairs(sets[setName]) do
		-- l = l + 1
	-- end
	-- return l
	return #entryLookUp[setName]
end

function fileSet_module.getRandomName(setName)
	return entryLookUp[setName][math.random(1, #entryLookUp[setName])]
end

return fileSet_module