local newFileSet_module = {}

local u_loadModuleFiles = require("modules/utils/loadModuleFiles")
local u_error = require("modules/utils/error")
local u_argParse = require("modules/utils/argParse")

-- order: #type(eg. bullet, item, melee)/name/data,more_data,moore

-- local TEMPLATES_FOLDER = "modules/file/templates"

local sets = {}
local functions = {}
local setLookUp = {}

local line = 1
local sn = ""
local en = ""

function newFileSet_module.loadFunctions(filePath)
	functions = u_loadModuleFiles(filePath, "lua", "template")
end

local function processArgs(data)
	if data:len() == 0 then
		return {nil}
	end
	local processedData = {}
	local pointer = 1
	while #data >= pointer do
		local isFunction, newPointerPos, functionName, functionArgs = data:find("^(%w+)%((.-)%),?", pointer)
		if isFunction then -- function
			local processedArgs = processArgs(functionArgs) -- TODO: implement the ":" separator
			if functions[functionName] then
				processedData[#processedData + 1] = functions[functionName](processedArgs, sn, en)
			else
				print("WARNING!\nFunction type "..functionName.." is not defined\nLine: "..line)
				processedData[#processedData + 1] = processedArgs
			end
			pointer = newPointerPos + 1
		else -- single
			local _, newPointerPos, singleArg = data:find('^([%w".]+),?', pointer)
			processedData[#processedData + 1] = u_argParse(singleArg)
			pointer = newPointerPos + 1
		end
	end
	return processedData
end

function newFileSet_module.load(fileName)
	if not love.filesystem.info(fileName) then
		u_error("loading a file set", "file not found!")
	end
	for line in love.filesystem.lines(fileName) do
		local doesExist, _, setName, entryName, data = line:find("^#(%w+)/(%w+)/(.*)$")
		sn = setName
		en = entryName
		if doesExist then
			local finalData = processArgs(data)
			-- insert finalized data
			if not sets[setName] then
				sets[setName] = {}
			end
			sets[setName][entryName] = finalData
			-- look up table
			if not setLookUp[setName] then
				setLookUp[setName] = {}
			end
			setLookUp[setName][#setLookUp[setName] + 1] = entryName
		end
		line = line + 1
	end
	line = 1
end

function newFileSet_module.get(setName, entryName)
	return sets[setName][entryName]
end

function newFileSet_module.getWhole(setName)
	return sets[setName]
end

function newFileSet_module.getLen(setName)
	return #setLookUp[setName]
end

function newFileSet_module.getRandomName(setName)
	return setLookUp[setName][math.random(1, #setLookUp[setName])]
end

return newFileSet_module