local settingFile_module = {}

local u_argParse = require("modules/utils/argParse")

--[[
	format:
	[setting name] = setting value
--]]

local SETTING_FILE_PATH = "config.txt"

local settingDefault = {}
local settings = {}

local isChanged = false

function settingFile_module.setDefaults(setDef)
	settingDefault = setDef
end

function settingFile_module.load()
	if not love.filesystem.getInfo(SETTING_FILE_PATH) then
		print("Setting file not found!\nIt will be created when the game is closed.")
		return
	end
	local lineCount = 1
	for line in love.filesystem.lines(SETTING_FILE_PATH) do
		local _, _, settingName, settingValue = string.find(line, "^%[([%w_]+)%]%s*=%s*(.+)$")
		repeat
			if settingName and settingValue then
				local parsedArg = u_argParse(settingValue)
				if parsedArg then
					settings[settingName] = parsedArg
					break
				end
			end
			error("Invalid data\nLine "..lineCount)
		until true
		lineCount = lineCount + 1
	end
	isChanged = false
end

function settingFile_module.save()
	if not isChanged then -- don't write if the settings didn't change during runtime
		return
	end
	local settingFile = love.filesystem.newFile(SETTING_FILE_PATH)
	settingFile:open("w")
	settingFile:setBuffer("line")
	settingFile:seek(0)
	for i, v in pairs(settings) do
		local t = type(v)
		if t == "string" then
			settingFile:write("["..i.."] = "..'"'..v..'"\n')
		else
			settingFile:write("["..i.."] = "..v.."\n")
		end
	end
	settingFile:close()
	settingFile:release()
end

function settingFile_module.getSetting(settingName)
	return settings[settingName]
end

function settingFile_module.setSetting(settingName, settingValue)
	if settings[settingName] ~= settingValue then
		settings[settingName] = settingValue
		isChanged = true
	end
end

function settingFile_module.setToDefault(settingName)
	if settingName then
		if settings[settingName] ~= settingDefault[settingName] then
			settings[settingName] = settingDefault[settingName]
			isChanged = true
		end
	else
		for i, v in pairs(settingDefault) do
			settings[i] = v
		end
		isChanged = true
	end
end

return settingFile_module