local inventoryManager_module = {}

local itemManager_module = require("modules/game/itemManager")

local u_error = require("modules/utils/error")

--[[
	inventory container:
	{itemIDs(8)}
--]]

local inventories = {}

local function _doesExist(id, m)
	if not inventories[id] then
		u_error(m, "inventory object does not exist!")
	end
end

local function nextAvailableSlot(id)
	for i = 1, 8 do
		if inventories[id][i] == 0 then
			return i
		end
	end
	return -1
end

function inventoryManager_module.new()
	local inventory = {0, 0, 0, 0, 0, 0, 0, 0} -- items
	local newID = #inventories + 1
	inventories[newID] = inventory
	return newID
end

function inventoryManager_module.destroy(id, removeItems)
	_doesExist(id, "destroying a inventory object")
	local newRemItm = removeItems or false
	if newRemItm then
		for i = 1, 8 do
			local itemID = inventories[id][i]
			if itemID ~= 0 then
				itemManager_module.destroy(itemID)
			end
		end
	end
	inventories[id] = nil
end

function inventoryManager_module.clear()
	for i, _ in pairs(inventories) do
		inventoryManager_module.destroy(i)
	end
end

function inventoryManager_module.getArray()
	return inventories
end

function inventoryManager_module.getEntry(id)
	return inventories[id]
end

function inventoryManager_module.isFull(id)
	_doesExist(id, "checking if a inventory object is full")
	if nextAvailableSlot(id) == -1 then
		return true
	end
	return false
end

function inventoryManager_module.getitemID(id, pointer)
	_doesExist(id, "getting a item ID from a inventory object")
	return inventories[id][pointer]
end

function inventoryManager_module.insertItem(id, pointer, position)
	_doesExist(id, "inserting a item object into a inventory object")
	local newPosition = position or nextAvailableSlot(id)
	if newPosition == -1 then
		return "full", -1
	end
	itemManager_module.removeFromPlayField(pointer)
	inventories[id][newPosition] = pointer
	return "ok", newPosition
end

function inventoryManager_module.destroyItem(id, position)
	_doesExist(id, "destroying a item object from a inventory object")
	local itemID = inventories[id][position]
	itemManager_module.destroy(itemID)
	inventories[id][position] = 0
end

function inventoryManager_module.ejectItem(id, position, x, y)
	_doesExist(id, "ejecting a item object from a inventory object")
	local itemID = inventories[id][position]
	itemManager_module.insertIntoPlayField(itemID, x, y)
	inventories[id][position] = 0
end

function inventoryManager_module.update()
	for inventoryID, v in pairs(inventories) do
		for i = 1, 8 do
			if v[i] ~= 0 and not itemManager_module.doesExist(v[i]) then
				v[i] = 0
			end
		end
	end
end

return inventoryManager_module