local meleeManager_module = {}

local entityManager_module = require("modules/entity/entityManager")
local playerManager_module = require("modules/game/playerManager")
local fileSet_module = require("modules/file/fileSet")
local sound_module = require("modules/sound/sound")

local u_error = require("modules/utils/error")
local u_decodeDirection = require("modules/utils/decodeDirection")

--[[
	melee container:
	{entityID, playerID, stayTime, damage, hitTime}
	
	#item/knife/sprite(knife),boundingBox(6,9,2,12),weapon,melee,knife,12,25,const,24
	sprite,boundingBox,itemType,weaponType,meleeType,cooldown,stayTime,damageType,firstNumber[,secondNumber]
--]]

local meleeObjects = {}

local function _doesExist(id, m)
	if not meleeObjects[id] then
		u_error(m, "melee object does not exist!")
	end
end

local directionLookup = {1, 1, 1, 3, 2, 3, 3, 2, 3}

function meleeManager_module.new(playerID, meleeWeaponEntry, direction)
	local melee = {}
	local playerEntityID = playerManager_module.getEntityID(playerID)
	local playerPosition = entityManager_module.getPosition(playerEntityID)
	local offsetX, offsetY = u_decodeDirection(direction, 16, -16, 0, -16, 16)
	local itemSet = nil
	if type(meleeWeaponEntry) == "table" then -- small hack until i find a way to get the fist's data into fileSet without the item spawner spawning it
		itemSet = {}
		itemSet[5] = meleeWeaponEntry[1] -- melee type
		itemSet[7] = meleeWeaponEntry[2] -- stay time
		itemSet[8] = meleeWeaponEntry[3] -- damage type
		itemSet[9] = meleeWeaponEntry[4] -- first number
		itemSet[10]= meleeWeaponEntry[5] -- second number
	else
		itemSet = fileSet_module.get("item", meleeWeaponEntry)
	end
	local meleeSet = fileSet_module.get("melee", itemSet[5])
	local entityID = entityManager_module.new(
		playerPosition[1] + offsetX, playerPosition[2] + offsetY, meleeSet[1][directionLookup[direction]]
	)
	entityManager_module.anchor(playerID, entityID)
	entityManager_module.setAnchorOffset(entityID, offsetX, offsetY)
	entityManager_module.flip(entityID, u_decodeDirection(direction, true, false, false, false, true))
	local boundingBoxes = meleeSet[directionLookup[direction] + 1]
	for i = 1, #boundingBoxes do
		entityManager_module.setBoundingBox(
			entityID, i, boundingBoxes[i][1], boundingBoxes[i][2], boundingBoxes[i][3], boundingBoxes[i][4]
		)
	end
	local damage = 0
	if itemSet[8] == "const" then
		damage = tonumber(itemSet[9])
	elseif itemSet[8] == "rand" then
		damage = math.random(tonumber(itemSet[9]), tonumber(itemSet[10]))
	end
	melee[1] = entityID -- entity ID
	melee[2] = playerID -- player ID
	melee[3] = tonumber(itemSet[7]) -- stay time
	melee[4] = damage -- damage
	melee[5] = 1 -- hitTime
	local newID = #meleeObjects + 1
	meleeObjects[newID] = melee
	return newID
end

function meleeManager_module.destroy(id)
	_doesExist(id, "destroying a melee object")
	entityManager_module.destroy(meleeObjects[id][1])
	meleeObjects[id] = nil
end

function meleeManager_module.clear()
	for i, _ in pairs(meleeObjects) do
		meleeManager_module.destroy(i)
	end
end

function meleeManager_module.getArray()
	return meleeObjects
end

function meleeManager_module.getEntry(id)
	return meleeObjects[id]
end

function meleeManager_module.update()
	for objID, v in pairs(meleeObjects) do
		-- player collision
		if v[5] > 0 then
			for playerID, playerObj in pairs(playerManager_module.getArray()) do
				if playerID ~= v[2] and entityManager_module.entityCollision(playerManager_module.getEntityID(playerID), v[1]) then
					sound_module.soundEffectsHelper("hit")
					playerManager_module.addHealth(playerID, -v[4])
				end
			end
			v[5] = v[5] - 1
		end
		-- stay time
		if v[3] <= 0 then
			meleeManager_module.destroy(objID)
		else
			v[3] = v[3] - 1
		end
	end
end

return meleeManager_module