local projectileManager_module = {}

local entityManager_module = require("modules/entity/entityManager")
local fileSet_module = require("modules/file/fileSet")
local playerManager_module = require("modules/game/playerManager")
local sound_module = require("modules/sound/sound")

local u_error = require("modules/utils/error")
local u_decodeDirection = require("modules/utils/decodeDirection")

--[[
	projectile container:
	{entityID, playerID, stayTime, damage, XSpeed, YSpeed}

	#item/gun/sprite(gun),boundingBox(4,13,2,12),weapon,range,gun,12,46,2,50,false,rand,4,11
	sprite,boundingBox,itemType,weaponType,projectileType,cooldown,stayTime,projectileSpeed,startingAmmo,autoToggle,damageType,
	firstNumber[,secondNumber]

	TODO:{
		automatic fire <weapon_itemType>
	}
--]]

local projectiles = {}

local function _doesExist(id, m)
	if not projectiles[id] then
		u_error(m, "projectile object does not exist!")
	end
end

local directionLookup = {1, 1, 1, 3, 2, 3, 3, 2, 3}

function projectileManager_module.new(playerID, projectileEntry, direction)
	local projectile = {}
	local itemEntry = fileSet_module.get("item", projectileEntry)
	local bulletEntry = fileSet_module.get("bullet", itemEntry[5])
	local playerEntityID = playerManager_module.getEntityID(playerID)
	local playerPosition = entityManager_module.getPosition(playerEntityID)
	local entityID = entityManager_module.new(playerPosition[1], playerPosition[2], bulletEntry[1][directionLookup[direction]])
	entityManager_module.flip(entityID, u_decodeDirection(direction, true, false, false, false, true))
	local speed = tonumber(itemEntry[8])
	local XSpeed, YSpeed = u_decodeDirection(direction, speed, -speed, 0, -speed, speed)
	entityManager_module.setForce(entityID, XSpeed, YSpeed)
	for i, v in pairs(bulletEntry[2]) do
		entityManager_module.setBoundingBox(entityID, i, v[1], v[2], v[3], v[4])
	end
	entityManager_module.setCollidable(entityID, false)
	local damage = 0
	if itemEntry[11] == "rand" then
		damage = math.random(tonumber(itemEntry[12]), tonumber(itemEntry[13]))
	elseif itemEntry[11] == "const" then
		damage = tonumber(itemEntry[12])
	end
	projectile[1] = entityID -- entity ID
	projectile[2] = playerID -- player ID
	projectile[3] = tonumber(itemEntry[7]) -- stay time
	projectile[4] = damage -- damage
	projectile[5] = XSpeed -- X speed
	projectile[6] = YSpeed -- Y speed
	local newID = #projectiles + 1
	projectiles[newID] = projectile
	return newID
end

function projectileManager_module.destroy(id)
	_doesExist(id, "destroying a projectile object")
	entityManager_module.destroy(projectiles[id][1])
	projectiles[id] = nil
end

function projectileManager_module.clear()
	for i, _ in pairs(projectiles) do
		projectileManager_module.destroy(i)
	end
end

function projectileManager_module.getArray()
	return projectiles
end

function projectileManager_module.getEntry(id)
	return projectiles[id]
end

function projectileManager_module.update()
	for objID, v in pairs(projectiles) do
		-- force
		entityManager_module.setForce(v[1], v[5], v[6])
		-- player collision
		for playerID, playerObj in pairs(playerManager_module.getArray()) do
			if v[2] ~= playerID and entityManager_module.entityCollision(playerManager_module.getEntityID(playerID), v[1]) then
				sound_module.soundEffectsHelper("hit")
				playerManager_module.addHealth(playerID, -v[4])
				v[3] = 0
			end
		end
		-- map collision
		if entityManager_module.mapCollision(v[1]) then
			v[3] = 0
		end
		-- stay time
		if v[3] > 0 then
			v[3] = v[3] - 1
		else
			projectileManager_module.destroy(objID)
		end
	end
end

return projectileManager_module
