local timerManager_module = {}

local u_error = require("modules/utils/error")

--[[
	timer container:
	{time, state, start time}
	
	states:
	0: running 1: paused 2: finished
--]]

local timers = {}

local function _doesExist(id, m)
	if not timers[id] then
		u_error(m, "timer object does not exist!")
	end
end


function timerManager_module.new(newTime, startPaused)
	local newTimeObj = {}
	newTimeObj[1] = newTime -- time
	newTimeObj[2] = startPaused and 1 or 0 -- state
	newTimeObj[3] = newTime -- start time
	local newID = #timers + 1
	timers[newID] = newTimeObj
	return newID
end

function timerManager_module.destroy(id)
	_doesExist(id, "destroying a timer object")
	timers[id] = nil
end

function timerManager_module.clear()
	for i, _ in pairs(timers) do
		return timerManager_module.destroy(i)
	end
end

function timerManager_module.getArray()
	return timers
end

function timerManager_module.getEntry(id)
	return timers[id]
end

function timerManager_module.getTime(id)
	_doesExist(id, "getting a time value from a timer object")
	return timers[id][1]
end

function timerManager_module.getTimeRounded(id)
	_doesExist(id, "getting a time value from a timer object")
	return math.floor(timers[id][1])
end

function timerManager_module.getState(id)
	_doesExist(id, "getting the state of a timer object")
	return timers[id][2]
end

function timerManager_module.setTime(id, newTime)
	_doesExist(id, "setting a time value from a timer object")
	timers[id][1] = newTime
	timers[id][2] = 1
	timers[id][3] = newTime
end

function timerManager_module.pause(id)
	_doesExist(id, "pausing a timer object")
	timers[id][2] = 1
end

function timerManager_module.resume(id)
	_doesExist(id, "resuming a timer object")
	timers[id][2] = 0
end

function timerManager_module.reset(id)
	_doesExist(id, "resetting a timer object")
	timers[id][1] = timers[id][3]
	timers[id][2] = 1
end

function timerManager_module.update(dt)
	for i, v in pairs(timers) do
		if v[2] == 0 then
			v[1] = math.max(v[1] - dt, 0.0)
			if v[1] <= 0.0 then
				v[2] = 2
			end
		end
	end
end

return timerManager_module