local mapCanvas_module = {}

local mapLoader_module = require("modules/map/mapLoader")
local mapTile_module = require("modules/map/mapTile")
local shaderManager_module = require("modules/graphics/shaderManager")

local BACKGROUND_TILE_IMAGE = love.graphics.newImage("res/inventory/background.png")
local INVENTORY_OVERLAY_IMAGE = love.graphics.newImage("res/inventory/inventory_overlay.png")

local map = love.graphics.newCanvas(16 * 16, 16 * 11)
local inventoryInterface = love.graphics.newSpriteBatch(BACKGROUND_TILE_IMAGE, 32, "static")
local inventoryOverlay = love.graphics.newCanvas(16 * 16, 16)
local textCanvas = love.graphics.newCanvas(16 * 16, 16 * 2)

local textStr = ""
local textRedraw = false

function mapCanvas_module.setText(txt)
	textStr = txt
	textRedraw = true
end

function mapCanvas_module.forceTextRedraw()
	textRedraw = true
end

function mapCanvas_module.renderMap()
	love.graphics.push("all")
	love.graphics.reset()
	love.graphics.setCanvas(map)
	local foreMap = mapLoader_module.getForeMap()
	local backMap = mapLoader_module.getBackMap()
	local foreTiles = mapTile_module.getForegroundTiles()
	local backTiles = mapTile_module.getBackgroundTiles()
	for y = 10, 0, -1 do
		for x = 15, 0, -1 do
			local tileNum = (x + 1) + y * 16
			love.graphics.draw(backTiles[backMap[tileNum]], 16 * x, 16 * y)
			if foreMap[tileNum] ~= "00" then
				love.graphics.draw(foreTiles[foreMap[tileNum]], 16 * x, 16 * y)
			end
		end
	end
	-- love.graphics.setCanvas()
	love.graphics.pop()
end

function mapCanvas_module.renderInventoryInterface()
	for x = 0, 15 do
		inventoryInterface:add(16 * x, 0)
		inventoryInterface:add(16 * x, 16)
	end
end

function mapCanvas_module.renderInventoryOverlay()
	love.graphics.setCanvas(inventoryOverlay)
	shaderManager_module.setShader("tint")
	shaderManager_module.sendColor("tint", {0, 0, 1}) -- player 1
	love.graphics.draw(INVENTORY_OVERLAY_IMAGE)
	shaderManager_module.sendColor("tint", {1, 0, 0}) -- player 2
	love.graphics.draw(INVENTORY_OVERLAY_IMAGE, 128, 0)
	shaderManager_module.setShader()
	love.graphics.setCanvas()
	inventoryOverlay = love.graphics.newImage(inventoryOverlay:newImageData())
end

function mapCanvas_module.drawText(posX, posY)
	if textRedraw then
		love.graphics.push("all")
		love.graphics.reset()
		love.graphics.setCanvas(textCanvas)
		love.graphics.setColor(0, 0, 0)
		love.graphics.clear()
		love.graphics.print(textStr)
		love.graphics.pop()
		textRedraw = false
	end
	love.graphics.draw(textCanvas, posX, posY)
end

function mapCanvas_module.getMapGraphic()
	return map
end

function mapCanvas_module.getInventoryInterface()
	return inventoryInterface
end

function mapCanvas_module.getInventoryOverlay()
	return inventoryOverlay
end

return mapCanvas_module