local shaderManager_module = {}

local u_loadModuleFiles = require("modules/utils/loadModuleFiles")

local SHADERS_FOLDER = "res/shaders"

local shaders = {}
local activeShader = nil
local activeShaderName = ""

function shaderManager_module.loadShaders(condition)
	shaders = u_loadModuleFiles(SHADERS_FOLDER, "glsl", "shader", function(filePath, fileName)
		local newFile = ""
		if love.filesystem.getInfo(filePath, "directory") then
			print("Hit condition path "..filePath)
			local cond = condition[fileName]
			if cond then
				local fin = cond[2][cond[1]]
				if fin then
					newFile = filePath.."/"..fin..".glsl"
				elseif cond[2]["default"] then
					newFile = filePath.."/"..cond[2]["default"]..".glsl"
				else
					print("Condition failed")
					return
				end
			else
				print("No condition found")
				return
			end
		else
			newFile = filePath..".glsl"
		end
		newFile = love.filesystem.newFile(newFile)
		local fileBits = newFile:read() -- file:read() returns two values, a string and a number
		print("Compiling shader "..filePath)
		return love.graphics.newShader(fileBits)
	end, true)
end

function shaderManager_module.setShader(shaderName)
	if shaderName and activeShaderName ~= shaderName then
		activeShader = shaders[shaderName]
		activeShaderName = shaderName
	end
	love.graphics.setShader(shaderName and activeShader or nil)
end

function shaderManager_module.send(...)
	activeShader:send(...)
end

function shaderManager_module.sendColor(...)
	activeShader:sendColor(...)
end

return shaderManager_module