local text_module = {}

local shaderManager_module = require("modules/graphics/shaderManager")

local u_createQuads = require("modules/utils/createQuads")

local SMALL_NUMBER_WIDTH = 4
local SMALL_NUMBER_HEGIHT = 6
local BIG_NUMBER_WIDTH = 7
local BIG_NUMBER_HEGIHT = 14
local SYMBOL_WIDTH = 8
local SYMBOL_HEGIHT = 8

local smallNumberText  = nil
local smallNumberQuads = nil
local bigNumberText  = nil
local bigNumberQuads = nil
local symbols  = nil
local symbolQuads = nil

local function startShader(c)
	shaderManager_module.setShader("tint")
	shaderManager_module.sendColor("tint", c or {0, 0, 0})
end

function text_module.prepare()
	smallNumberText  = love.graphics.newImage("res/char/smallNumbers.png")
	smallNumberQuads = u_createQuads(SMALL_NUMBER_WIDTH, SMALL_NUMBER_HEGIHT, smallNumberText:getDimensions())
	bigNumberText  = love.graphics.newImage("res/char/bigNumbers.png")
	bigNumberQuads = u_createQuads(BIG_NUMBER_WIDTH, BIG_NUMBER_HEGIHT, bigNumberText:getDimensions())
	symbols  = love.graphics.newImage("res/char/symbols.png")
	symbolQuads = u_createQuads(SYMBOL_WIDTH, SYMBOL_HEGIHT, symbols:getDimensions())
end

function text_module.smallNumber(number, x, y, color)
	startShader(color)
	local drawX = x
	local textString = tostring(number)
	for i = 1, #textString do
		local c = tonumber(string.sub(textString, i, i)) + 1
		love.graphics.draw(smallNumberText, smallNumberQuads[c], drawX, y)
		drawX = drawX + SMALL_NUMBER_WIDTH
	end
	shaderManager_module.setShader()
end

function text_module.smallNumberRev(number, x, y, color)
	startShader(color)
	local drawX = x + SMALL_NUMBER_WIDTH - 1
	local textString = tostring(number)
	for i = #textString, 1, -1 do
		local c = tonumber(string.sub(textString, i, i)) + 1
		love.graphics.draw(smallNumberText, smallNumberQuads[c], drawX, y)
		drawX = drawX - SMALL_NUMBER_WIDTH
	end
	shaderManager_module.setShader()
end

function text_module.bigNumber(number, x, y, color)
	startShader(color)
	local drawX = x
	local textString = tostring(number)
	for i = 1, #textString do
		local c = tonumber(string.sub(textString, i, i)) + 1
		love.graphics.draw(bigNumberText, bigNumberQuads[c], drawX, y)
		drawX = drawX + BIG_NUMBER_WIDTH
	end
	shaderManager_module.setShader()
end

function text_module.symbol(sel, x, y)
	love.graphics.draw(symbols, symbolQuads[sel], x, y)
end

return text_module