local mapLoader_module = {}

local MAP_SIZE = 16 * 11

local foreMap = {}
local backMap = {}
local specMap = {}
local player1Pos = {0, 0}
local player2Pos = {0, 0}
local name = ""
local timeMinute = 0
local timeSecond = 0

local function BLANK() -- BLAAAAAAAAAAAAAAAAAANNNNNNNNNNNNNNNNNNNNNNK!!!
	foreMap = {}
	backMap = {}
	specMap = {}
	player1Pos = {0, 0}
	player2Pos = {0, 0}
	name = ""
	timeMinute = 0
	timeSecond = 0
end

function mapLoader_module.loadMap(mapName)
	local nepName = "maps/"..mapName
	if not love.filesystem.getInfo(nepName) then
		error("Map '"..mapName.."' not found!")
	end
	nepName = nepName.."/data.txt"
	if not love.filesystem.getInfo(nepName) then
		error("data.txt is not found in map "..mapName)
	end
	BLANK()
	name = mapName
	local isLoadingAtt = true
	local tileMap = {}
	local maps = {}
	local loadLines = true
	for line in love.filesystem.lines(nepName) do
		if isLoadingAtt then -- map attributes
			-- load stuff
			local attributes = {}
			string.gsub(line, "(%w+)%s*=%s*(.-):", function(attName, attValues)
				local vars = {}
				string.gsub(attValues, '([%w"]+)%s*,?', function(v)
					if not string.find(v, '"%w+"') then
						v = tonumber(v)
					end
					vars[#vars + 1] = v
				end)
				attributes[attName] = vars
			end)
			-- set variables
			if attributes["time"] then
				timeMinute = attributes["time"][1]
				timeSecond = attributes["time"][2]
			end
			isLoadingAtt = false
		else -- map data
			if loadLines then
				string.gsub(line, "(%w%w)", function(tile)
					tileMap[#tileMap + 1] = tile
				end)
				if #tileMap >= MAP_SIZE then
					maps[#maps + 1] = tileMap
					tileMap = {}
					loadLines = false
				end
			else
				loadLines = true
			end
		end
	end
	foreMap = maps[1]
	backMap = maps[2]
	specMap = maps[3]
	local pointer = {0, 0}
	for i = 1, #specMap do
		if specMap[i] == "p1" then -- player 1 spawn
			player1Pos = {16 * pointer[1], 16 * pointer[2]}
		elseif specMap[i] == "p2" then -- player 2 spawn
			player2Pos = {16 * pointer[1], 16 * pointer[2]}
		end
		if pointer[1] == 16 then
			pointer[2] = pointer[2] + 1
			pointer[1] = 0
		end
		pointer[1] = pointer[1] + 1
	end
end

function mapLoader_module.getForeMap()
	return foreMap
end

function mapLoader_module.getBackMap()
	return backMap
end

function mapLoader_module.getSpecMap()
	return specMap
end

function mapLoader_module.getForeTile(x, y)
	return foreMap[16 * (y - 1) + x]
end

function mapLoader_module.getBackTile(x, y)
	return backMap[16 * (y - 1) + x]
end

function mapLoader_module.getSpecTile(x, y)
	return specMap[16 * (y - 1) + x]
end

function mapLoader_module.getMapTime()
	return timeMinute, timeSecond
end

function mapLoader_module.getMapName()
	return name
end

function mapLoader_module.getPlayerPos(sel)
	if sel == 1 then
		return player1Pos
	end
	return player2Pos
end

return mapLoader_module