local mapTile_module = {} -- MAYBE: don't delete tiles that appears in the previous map and the next map

local mapLoader_module = require("modules/map/mapLoader")

local foregroundTiles = {}
local backgroundTiles = {}

function mapTile_module.addForeTile(name, tilePath)
	foregroundTiles[name] = love.graphics.newImage(tilePath)
end
function mapTile_module.addBackTile(name, tilePath)
	backgroundTiles[name] = love.graphics.newImage(tilePath)
end

local function getUqiTil(map)
	local tilesToLoad = {}
	local tileTag = {}
	for i = 1, #map do
		tileTag[map[i]] = 0
	end
	for i in pairs(tileTag) do
		tilesToLoad[#tilesToLoad + 1] = i
	end
	return tilesToLoad
end

function mapTile_module.refreshTiles()
	for i = 1, #foregroundTiles do
		foregroundTiles[i]:release()
	end
	foregroundTiles = {}
	for i = 1, #backgroundTiles do
		backgroundTiles[i]:release()
	end
	backgroundTiles = {}
	-- identify the tiles that need to be loaded
	local foreTiles = getUqiTil(mapLoader_module.getForeMap())
	local backTiles = getUqiTil(mapLoader_module.getBackMap())
	-- load the tiles
	local mapName = mapLoader_module.getMapName()
	-- foreground tiles
	for i = 1, #foreTiles do
		local tile = foreTiles[i]
		local tilePath = "maps/"..mapName.."/"..tile..".png"
		if love.filesystem.getInfo(tilePath) then -- custom tiles
			mapTile_module.addForeTile(tile, tilePath)
		else -- default tiles
			tilePath = "tiles/"..tile..".png"
			if love.filesystem.getInfo(tilePath) then
				mapTile_module.addForeTile(tile, tilePath)
			end
		end
	end
	-- background tiles
	for i = 1, #backTiles do
		local tile = backTiles[i]
		local tilePath = "maps/"..mapName.."/"..tile..".png"
		if love.filesystem.getInfo(tilePath) then -- custom tiles
			mapTile_module.addBackTile(tile, tilePath)
		else -- default tiles
			tilePath = "bgTiles/"..tile..".png"
			if love.filesystem.getInfo(tilePath) then
				mapTile_module.addBackTile(tile, tilePath)
			end
		end
	end
end

function mapTile_module.getForegroundTiles()
	return foregroundTiles
end

function mapTile_module.getBackgroundTiles()
	return backgroundTiles
end

return mapTile_module